searchNodes=[{"doc":"A bitwise radix tree for prefix based matching on bitstring keys of any length. Radix provides a radix tree whose radius is 2, has path-compression and no one-way branching. Entries consist of {key, value}-pairs whose insertion/deletion is always based on an exact key-match. Retrieval can be either exact or is based on a longest prefix match. Examples iex&gt; t = new ( ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1/24&quot; ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 , 0 :: 6 &gt;&gt; , &quot;1.1.1.0/30&quot; ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; ) ...&gt; |&gt; put ( &lt;&lt; 255 &gt;&gt; , &quot;255/8&quot; ) iex&gt; iex&gt; iex&gt; # longest prefix match iex&gt; iex&gt; lookup ( t , &lt;&lt; 1 , 1 , 1 , 255 &gt;&gt; ) { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } iex&gt; iex&gt; iex&gt; # more specific matches (includes search key if present) iex&gt; iex&gt; more ( t , &lt;&lt; 1 , 1 , 1 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 1 , 0 :: size ( 6 ) &gt;&gt; , &quot;1.1.1.0/30&quot; } , { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1/24&quot; } , { &lt;&lt; 1 , 1 , 1 , 1 :: size ( 1 ) &gt;&gt; , &quot;1.1.1.128/25&quot; } ] iex&gt; iex&gt; iex&gt; # less specific matches (includes search key if present) iex&gt; iex&gt; less ( t , &lt;&lt; 1 , 1 , 1 , 3 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 1 , 0 :: size ( 6 ) &gt;&gt; , &quot;1.1.1.0/30&quot; } , { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1/24&quot; } ] iex&gt; iex&gt; # exact match iex&gt; get ( t , &lt;&lt; 1 , 1 , 1 , 0 :: 6 &gt;&gt; ) { &lt;&lt; 1 , 1 , 1 , 0 :: 6 &gt;&gt; , &quot;1.1.1.0/30&quot; } iex&gt; iex&gt; get ( t , &lt;&lt; 1 , 1 , 1 , 0 &gt;&gt; ) nil iex&gt; iex&gt; dot ( t ) |&gt; ( &amp; File . write ( &quot;assets/readme.dot&quot; , &amp;1 ) ) . ( ) The radix tree above looks something like this: The tree is represented by two types of nodes: internal node , as a {bit, left, right} -tuple, and leaf node , which is either nil or a non-empty list of {key,value} -pairs The bit denotes the bit position to check in a key during a tree traversal, where 0 means go left and 1 means go right . A bit beyond a key 's length is considered to be 0 . Path-compression means not all bits are checked during tree traversal, only those that differentiate the keys stored below the current internal node. Hence, branches are formed as keys with different patterns are stored in the tree. The leaf node can have a list of {key, value} -pairs where all longer keys have all shorter keys as their prefix. In other words, they all agree on the bits that were checked to arrive at that node. The key is stored alongside the value since, due to path-compression, a final match is needed to ensure a correct match. Hence, retrieval functions return the {key, value} -pair, rather than just the value , since the stored key is not always equal to the given search key (e.g. when doing a longest prefix match). Since binaries are bitstrings too, they work as well: iex&gt; t = new ( [ { &quot;A.new&quot; , &quot;new&quot; } , { &quot;A.newer&quot; , &quot;newer&quot; } , { &quot;B.newest&quot; , &quot;newest&quot; } ] ) iex&gt; more ( t , &quot;A.&quot; ) |&gt; Enum . reverse ( ) [ { &quot;A.new&quot; , &quot;new&quot; } , { &quot;A.newer&quot; , &quot;newer&quot; } ] # iex&gt; lookup ( t , &quot;A.newest&quot; ) { &quot;A.new&quot; , &quot;new&quot; } # iex&gt; more ( t , &quot;C.&quot; ) [ ]","ref":"Radix.html","title":"Radix","type":"module"},{"doc":"Counts the number of entries by traversing given tree . Example iex&gt; new ( [ { &lt;&lt; &gt;&gt; , nil } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } ] ) ...&gt; |&gt; count 3","ref":"Radix.html#count/1","title":"Radix.count/1","type":"function"},{"doc":"Delete the entry from the tree for a specific key using an exact match. If key does not exist, the tree is returned unchanged. Example iex&gt; elms = [ { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] iex&gt; t = new ( elms ) iex&gt; t { 0 , { 23 , [ { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 &gt;&gt; , 16 } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] } , nil } # iex&gt; delete ( t , &lt;&lt; 1 , 1 , 0 &gt;&gt; ) { 0 , { 23 , [ { &lt;&lt; 1 , 1 &gt;&gt; , 16 } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] } , nil }","ref":"Radix.html#delete/2","title":"Radix.delete/2","type":"function"},{"doc":"Given a tree, returns a list of lines describing the tree as a graphviz digraph. Options include: :label , defaults to &quot;radix&quot;) :labelloc , defaults to &quot;t&quot; :rankdir , defaults to &quot;TB&quot; :ranksep , defaults to &quot;0.5 equally&quot; :rootcolor , defaults to &quot;orange&quot; :nodecolor , defaults to &quot;yellow&quot; :leafcolor , defaults to &quot;green&quot; :kv_tostr , defaults to an internal function that converts key to dotted decimal string (cidr style) If supplied via :kv_tostr , the function's signature must be ({ key/0 , value/0 }) :: String.t/0 and where the resulting string must be HTML-escaped. See html-entities . Works best for smaller trees. Example iex&gt; t = new ( ) ...&gt; |&gt; put ( &lt;&lt; 0 , 0 &gt;&gt; , &quot;left&quot; ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 :: 1 &gt;&gt; , &quot;left&quot; ) ...&gt; |&gt; put ( &lt;&lt; 128 , 0 &gt;&gt; , &quot;right&quot; ) iex&gt; g = dot ( t , label : &quot;example&quot; ) [ &quot;digraph Radix { \\n labelloc= \\&quot; t \\&quot; ; \\n label= \\&quot; example \\&quot; ; \\n rankdir= \\&quot; TB \\&quot; ; \\n ranksep= \\&quot; 0.5 equally \\&quot; ; \\n &quot; , &quot;N4 [label=&lt; \\n &lt;TABLE BORDER= \\&quot; 0 \\&quot; CELLBORDER= \\&quot; 1 \\&quot; CELLSPACING= \\&quot; 0 \\&quot; &gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; N4 \\&quot; BGCOLOR= \\&quot; green \\&quot; &gt;leaf&lt;/TD&gt;&lt;/TR&gt; \\n &lt;TR&gt;&lt;TD&gt;128.0/16&lt;/TD&gt;&lt;/TR&gt; \\n &lt;/TABLE&gt; \\n &gt;, shape= \\&quot; plaintext \\&quot; ]; \\n &quot; , &quot;N2 [label=&lt; \\n &lt;TABLE BORDER= \\&quot; 0 \\&quot; CELLBORDER= \\&quot; 1 \\&quot; CELLSPACING= \\&quot; 0 \\&quot; &gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; N2 \\&quot; BGCOLOR= \\&quot; green \\&quot; &gt;leaf&lt;/TD&gt;&lt;/TR&gt; \\n &lt;TR&gt;&lt;TD&gt;1.1.128/17&lt;/TD&gt;&lt;/TR&gt; \\n &lt;/TABLE&gt; \\n &gt;, shape= \\&quot; plaintext \\&quot; ]; \\n &quot; , &quot;N1 [label=&lt; \\n &lt;TABLE BORDER= \\&quot; 0 \\&quot; CELLBORDER= \\&quot; 1 \\&quot; CELLSPACING= \\&quot; 0 \\&quot; &gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; N1 \\&quot; BGCOLOR= \\&quot; green \\&quot; &gt;leaf&lt;/TD&gt;&lt;/TR&gt; \\n &lt;TR&gt;&lt;TD&gt;0.0/16&lt;/TD&gt;&lt;/TR&gt; \\n &lt;/TABLE&gt; \\n &gt;, shape= \\&quot; plaintext \\&quot; ]; \\n &quot; , &quot;N3:R -&gt; N2; \\n &quot; , &quot;N3:L -&gt; N1; \\n &quot; , &quot;N3 [label=&lt; \\n &lt;TABLE BORDER= \\&quot; 0 \\&quot; CELLBORDER= \\&quot; 1 \\&quot; CELLSPACING= \\&quot; 0 \\&quot; &gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; N3 \\&quot; COLSPAN= \\&quot; 2 \\&quot; BGCOLOR= \\&quot; yellow \\&quot; &gt;bit 7&lt;/TD&gt;&lt;/TR&gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; L \\&quot; &gt;0&lt;/TD&gt;&lt;TD PORT= \\&quot; R \\&quot; &gt;1&lt;/TD&gt;&lt;/TR&gt; \\n &lt;/TABLE&gt; \\n &gt;, shape= \\&quot; plaintext \\&quot; ]; \\n &quot; , &quot;N5:R -&gt; N4; \\n &quot; , &quot;N5:L -&gt; N3; \\n &quot; , &quot;N5 [label=&lt; \\n &lt;TABLE BORDER= \\&quot; 0 \\&quot; CELLBORDER= \\&quot; 1 \\&quot; CELLSPACING= \\&quot; 0 \\&quot; &gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; N5 \\&quot; COLSPAN= \\&quot; 2 \\&quot; BGCOLOR= \\&quot; orange \\&quot; &gt;bit 0&lt;/TD&gt;&lt;/TR&gt; \\n &lt;TR&gt;&lt;TD PORT= \\&quot; L \\&quot; &gt;0&lt;/TD&gt;&lt;TD PORT= \\&quot; R \\&quot; &gt;1&lt;/TD&gt;&lt;/TR&gt; \\n &lt;/TABLE&gt; \\n &gt;, shape= \\&quot; plaintext \\&quot; ]; \\n &quot; , &quot;}&quot; ] iex&gt; File . write ( &quot;assets/example.dot&quot; , g ) :ok which, after converting with dot , yields the following image:","ref":"Radix.html#dot/2","title":"Radix.dot/2","type":"function"},{"doc":"Drops the given keys from the radix tree using an exact match. Any key 's that don't exist in the tree , are ignored. Example iex&gt; elms = [ { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] iex&gt; t = new ( elms ) iex&gt; t { 0 , { 23 , [ { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 &gt;&gt; , 16 } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] } , nil } # iex&gt; drop ( t , [ &lt;&lt; 1 , 1 &gt;&gt; , &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; ] ) { 0 , [ { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } ] , nil }","ref":"Radix.html#drop/2","title":"Radix.drop/2","type":"function"},{"doc":"Say whether given tree is empty or not. Example iex&gt; new ( ) |&gt; empty? ( ) true","ref":"Radix.html#empty?/1","title":"Radix.empty?/1","type":"function"},{"doc":"Fetches the key,value-pair for a specific key in the given tree . Returns {:ok, {key, value}} or :error when key is not in the tree . By default an exact match is used, specify match: :lpm to fetch based on a longest prefix match. Example iex&gt; t = new ( [ { &lt;&lt; &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 1 , 1 &gt;&gt; , 2 } ] ) iex&gt; fetch ( t , &lt;&lt; 1 , 1 &gt;&gt; ) { :ok , { &lt;&lt; 1 , 1 &gt;&gt; , 2 } } iex&gt; iex&gt; fetch ( t , &lt;&lt; 2 &gt;&gt; ) :error iex&gt; fetch ( t , &lt;&lt; 2 &gt;&gt; , match : :lpm ) { :ok , { &lt;&lt; &gt;&gt; , 0 } }","ref":"Radix.html#fetch/3","title":"Radix.fetch/3","type":"function"},{"doc":"Fetches the key,value-pair for a specific key in the given tree . Returns the {key, value} -pair itself, or raises a KeyError if key is not in the tree . By default an exact match is used, specify match: :lpm to fetch based on a longest prefix match. Example iex&gt; t = new ( [ { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 1 , 1 &gt;&gt; , 2 } ] ) iex&gt; fetch! ( t , &lt;&lt; 1 , 1 &gt;&gt; ) { &lt;&lt; 1 , 1 &gt;&gt; , 2 } iex&gt; iex&gt; fetch! ( t , &lt;&lt; 2 &gt;&gt; ) ** (KeyError) key not found &lt;&lt;0b10&gt;&gt; iex&gt; iex&gt; fetch! ( t , &lt;&lt; 1 , 1 , 1 &gt;&gt; , match : :lpm ) { &lt;&lt; 1 , 1 &gt;&gt; , 2 }","ref":"Radix.html#fetch!/3","title":"Radix.fetch!/3","type":"function"},{"doc":"Get the key,value-pair whose key equals the given search key . If key is not a bitstring or not present in the radix tree, default is returned. If default is not provided, nil is used. Example iex&gt; elements = [ { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] iex&gt; t = new ( elements ) iex&gt; get ( t , &lt;&lt; 1 , 1 , 1 &gt;&gt; ) { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 24 } iex&gt; get ( t , &lt;&lt; 1 , 1 &gt;&gt; ) { &lt;&lt; 1 , 1 &gt;&gt; , 16 } iex&gt; get ( t , &lt;&lt; 1 , 1 , 0 :: 1 &gt;&gt; ) nil iex&gt; get ( t , &lt;&lt; 1 , 1 , 0 :: 1 &gt;&gt; , &quot;oops&quot; ) &quot;oops&quot;","ref":"Radix.html#get/3","title":"Radix.get/3","type":"function"},{"doc":"Returns all keys from the radix tree . Example iex&gt; t = new ( [ ...&gt; { &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , &quot;1.1.1.0/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } , ...&gt; { &lt;&lt; 3 &gt;&gt; , &quot;3.0.0.0/8&quot; } , ...&gt; ] ) iex&gt; iex&gt; keys ( t ) [ &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , &lt;&lt; 1 , 1 , 1 &gt;&gt; , &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &lt;&lt; 3 &gt;&gt; ]","ref":"Radix.html#keys/1","title":"Radix.keys/1","type":"function"},{"doc":"Returns all key,value-pairs whose key is a prefix for the given search key . Collects key,value-pairs where the stored key is the same or less specific. Example iex&gt; elements = [ ...&gt; { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , ...&gt; { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , ...&gt; { &lt;&lt; 1 , 1 , 0 , 0 &gt;&gt; , 32 } , ...&gt; { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ...&gt; ] iex&gt; t = new ( elements ) iex&gt; iex&gt; less ( t , &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } , { &lt;&lt; 1 , 1 &gt;&gt; , 16 } ] # iex&gt; less ( t , &lt;&lt; 1 , 1 , 0 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 &gt;&gt; , 16 } ] # iex&gt; less ( t , &lt;&lt; 2 , 2 &gt;&gt; ) [ ]","ref":"Radix.html#less/2","title":"Radix.less/2","type":"function"},{"doc":"Get the key,value-pair whose key is the longest prefix of key . Returns {key, value} or nil if there was no match. Example iex&gt; elms = [ { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 , 0 , 0 :: 1 &gt;&gt; , 25 } ] iex&gt; t = new ( elms ) iex&gt; lookup ( t , &lt;&lt; 1 , 1 , 0 , 127 &gt;&gt; ) { &lt;&lt; 1 , 1 , 0 , 0 :: 1 &gt;&gt; , 25 } # iex&gt; lookup ( t , &lt;&lt; 1 , 1 , 0 , 128 &gt;&gt; ) { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } # iex&gt; lookup ( t , &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; ) { &lt;&lt; 1 , 1 &gt;&gt; , 16 } # iex&gt; lookup ( t , &lt;&lt; 2 , 2 , 2 , 2 &gt;&gt; ) nil","ref":"Radix.html#lookup/2","title":"Radix.lookup/2","type":"function"},{"doc":"Merges two radix trees into one. Adds all key,value-pairs of tree2 to tree1 , overwriting any existing entries. Example iex&gt; tree1 = new ( [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } ] ) iex&gt; tree2 = new ( [ { &lt;&lt; 0 &gt;&gt; , nil } , { &lt;&lt; 2 &gt;&gt; , 2 } ] ) iex&gt; merge ( tree1 , tree2 ) ...&gt; |&gt; to_list ( ) [ { &lt;&lt; 0 &gt;&gt; , nil } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } ]","ref":"Radix.html#merge/2","title":"Radix.merge/2","type":"function"},{"doc":"Merges two radix trees into one. Adds all key,value-pairs of tree2 to tree1 , resolving conflicts through given fun . Its arguments are the conflicting key/0 and the value/0 found in tree1 and the value/0 found in tree2 . Example # keep values of tree1, like merge(tree2, tree1) iex&gt; tree1 = new ( [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } ] ) iex&gt; tree2 = new ( [ { &lt;&lt; 0 &gt;&gt; , nil } , { &lt;&lt; 2 &gt;&gt; , 2 } ] ) iex&gt; merge ( tree1 , tree2 , fn _k , v1 , _v2 -&gt; v1 end ) ...&gt; |&gt; to_list ( ) [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } ]","ref":"Radix.html#merge/3","title":"Radix.merge/3","type":"function"},{"doc":"Returns all key,value-pairs where the given search key is a prefix for a stored key. Collects key,value-pairs where the stored key is the same or more specific. Example iex&gt; elements = [ ...&gt; { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , ...&gt; { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , ...&gt; { &lt;&lt; 1 , 1 , 0 , 0 &gt;&gt; , 32 } , ...&gt; { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ...&gt; ] iex&gt; t = new ( elements ) iex&gt; iex&gt; more ( t , &lt;&lt; 1 , 1 , 0 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 0 , 0 &gt;&gt; , 32 } , { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } ] # iex&gt; more ( t , &lt;&lt; 1 , 1 , 1 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] # iex&gt; more ( t , &lt;&lt; 2 &gt;&gt; ) [ ]","ref":"Radix.html#more/2","title":"Radix.more/2","type":"function"},{"doc":"Return a new, empty radix tree. Example iex&gt; new ( ) { 0 , nil , nil }","ref":"Radix.html#new/0","title":"Radix.new/0","type":"function"},{"doc":"Return a new radix tree, initialized using given list of { key , value }-pairs. Example iex&gt; elements = [ { &lt;&lt; 1 , 1 &gt;&gt; , 16 } , { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } , { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } ] iex&gt; new ( elements ) { 0 , { 23 , [ { &lt;&lt; 1 , 1 , 0 &gt;&gt; , 24 } , { &lt;&lt; 1 , 1 &gt;&gt; , 16 } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , 32 } ] } , nil }","ref":"Radix.html#new/1","title":"Radix.new/1","type":"function"},{"doc":"Removes the value associated with key and returns the matched key,value-pair and the new tree. Options include: default: value , returned if key could not be matched (defaults to nil) match: :lpm , specifies a longest prefix match instead of an exact match If given search key was not matched, the tree is unchanged and the key,value-pair will be the search key and the default value. Examples # pop an existing element iex&gt; new ( [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } ] ) ...&gt; |&gt; pop ( &lt;&lt; 1 &gt;&gt; ) { { &lt;&lt; 1 &gt;&gt; , 1 } , { 0 , { 6 , [ { &lt;&lt; 0 &gt;&gt; , 0 } ] , [ { &lt;&lt; 2 &gt;&gt; , 2 } ] } , nil } } # pop non-existing, using a default iex&gt; new ( [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } ] ) ...&gt; |&gt; pop ( &lt;&lt; 3 &gt;&gt; , default : :notfound ) { { &lt;&lt; 3 &gt;&gt; , :notfound } , { 0 , { 6 , { 7 , [ { &lt;&lt; 0 &gt;&gt; , 0 } ] , [ { &lt;&lt; 1 &gt;&gt; , 1 } ] } , [ { &lt;&lt; 2 &gt;&gt; , 2 } ] } , nil } } # pop using longest prefix match iex&gt; t = new ( [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } , { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } ] ) iex&gt; pop ( t , &lt;&lt; 1 , 1 , 1 , 255 &gt;&gt; , match : :lpm ) { { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } , { 0 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } ] , nil } }","ref":"Radix.html#pop/3","title":"Radix.pop/3","type":"function"},{"doc":"Stores the key,value-pairs from elements in the radix tree . Any existing key 's will have their value 's replaced. Examples iex&gt; elements = [ { &lt;&lt; 1 , 1 &gt;&gt; , &quot;1.1.0.0/16&quot; } , { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.1&quot; } ] iex&gt; new ( ) |&gt; put ( elements ) { 0 , { 23 , [ { &lt;&lt; 1 , 1 &gt;&gt; , &quot;1.1.0.0/16&quot; } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.1&quot; } ] } , nil }","ref":"Radix.html#put/2","title":"Radix.put/2","type":"function"},{"doc":"Stores the key , value -pair under key in the radix tree . Any existing key will have its value replaced. Examples iex&gt; t = new ( ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 &gt;&gt; , &quot;1.1.0.0/16&quot; ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &quot;x.x.x.x&quot; ) iex&gt; t { 0 , { 23 , [ { &lt;&lt; 1 , 1 &gt;&gt; , &quot;1.1.0.0/16&quot; } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &quot;x.x.x.x&quot; } ] } , nil } # iex&gt; put ( t , &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.1&quot; ) { 0 , { 23 , [ { &lt;&lt; 1 , 1 &gt;&gt; , &quot;1.1.0.0/16&quot; } ] , [ { &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.1&quot; } ] } , nil }","ref":"Radix.html#put/3","title":"Radix.put/3","type":"function"},{"doc":"Invokes fun for each key,value-pair in the radix tree with the accumulator. The initial value of the accumulator is acc . The function is invoked for each key,value-pair in the radix tree with the accumulator in a depth-first fashion. The result returned by the function is used as the accumulator for the next iteration. The function returns the last accumulator. fun 's signature is ( key/0 , value/0 , acc/0 ) -&gt; acc/0 . Example iex&gt; t = new ( [ ...&gt; { &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , &quot;1.1.1.0/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } , ...&gt; { &lt;&lt; 3 &gt;&gt; , &quot;3.0.0.0/8&quot; } , ...&gt; ] ) iex&gt; iex&gt; # get values iex&gt; iex&gt; f = fn _key , value , acc -&gt; [ value | acc ] end iex&gt; reduce ( t , [ ] , f ) |&gt; Enum . reverse ( ) [ &quot;1.1.1.0/25&quot; , &quot;1.1.1.0/24&quot; , &quot;1.1.1.128/25&quot; , &quot;3.0.0.0/8&quot; ]","ref":"Radix.html#reduce/3","title":"Radix.reduce/3","type":"function"},{"doc":"Extract key,value-pairs from tree into a new radix tree. Returns the new tree and the old tree with the key,value-pairs removed. By default an exact match is used, specify match: :lpm to match based on a longest prefix match. If none of the given keys match, the new tree will be empty and the old tree unchanged. Examples iex&gt; tree = new ( [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } , { &lt;&lt; 3 &gt;&gt; , 3 } ] ) iex&gt; { t1 , t2 } = split ( tree , [ &lt;&lt; 0 &gt;&gt; , &lt;&lt; 2 &gt;&gt; ] ) iex&gt; iex&gt; keys ( t1 ) [ &lt;&lt; 0 &gt;&gt; , &lt;&lt; 2 &gt;&gt; ] iex&gt; iex&gt; keys ( t2 ) [ &lt;&lt; 1 &gt;&gt; , &lt;&lt; 3 &gt;&gt; ] iex&gt; tree = new ( [ { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } , { &lt;&lt; 3 &gt;&gt; , 3 } ] ) iex&gt; { t1 , t2 } = split ( tree , [ &lt;&lt; 0 , 0 &gt;&gt; , &lt;&lt; 2 , 0 &gt;&gt; ] , match : :lpm ) iex&gt; iex&gt; keys ( t1 ) [ &lt;&lt; 0 &gt;&gt; , &lt;&lt; 2 &gt;&gt; ] iex&gt; iex&gt; keys ( t2 ) [ &lt;&lt; 1 &gt;&gt; , &lt;&lt; 3 &gt;&gt; ]","ref":"Radix.html#split/3","title":"Radix.split/3","type":"function"},{"doc":"Return a new tree with all the key,value-pairs whose key are in keys . If a key in keys does not exist in tree , it is ignored. By default keys are matched exactly, use the option match: :lpm to use longest prefix matching. Examples iex&gt; new ( [ { &lt;&lt; &gt;&gt; , nil } , { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 128 &gt;&gt; , 128 } , { &lt;&lt; 255 &gt;&gt; , 255 } ] ) ...&gt; |&gt; take ( [ &lt;&lt; &gt;&gt; , &lt;&lt; 1 &gt;&gt; , &lt;&lt; 255 &gt;&gt; ] ) ...&gt; |&gt; to_list ( ) [ { &lt;&lt; &gt;&gt; , nil } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 255 &gt;&gt; , 255 } ] # longest prefix match iex&gt; new ( [ { &lt;&lt; &gt;&gt; , nil } , { &lt;&lt; 0 &gt;&gt; , 0 } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 128 &gt;&gt; , 128 } , { &lt;&lt; 255 &gt;&gt; , 255 } ] ) ...&gt; |&gt; take ( [ &lt;&lt; 2 , 2 , 2 , 2 &gt;&gt; , &lt;&lt; 1 , 1 , 1 , 1 &gt;&gt; , &lt;&lt; 255 , 255 , 0 , 0 &gt;&gt; ] , match : :lpm ) ...&gt; |&gt; to_list ( ) [ { &lt;&lt; &gt;&gt; , nil } , { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 255 &gt;&gt; , 255 } ]","ref":"Radix.html#take/3","title":"Radix.take/3","type":"function"},{"doc":"Return all key,value-pairs as a flat list. Example iex&gt; tree = new ( [ ...&gt; { &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , &quot;1.1.1.0/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } , ...&gt; { &lt;&lt; 3 &gt;&gt; , &quot;3.0.0.0/8&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } ...&gt; ] ) iex&gt; to_list ( tree ) [ { &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , &quot;1.1.1.0/25&quot; } , { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } , { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } , { &lt;&lt; 3 &gt;&gt; , &quot;3.0.0.0/8&quot; } ]","ref":"Radix.html#to_list/1","title":"Radix.to_list/1","type":"function"},{"doc":"Lookup given search key in tree and update the value of matched key with the given function. If key has a longest prefix match in tree then its value is passed to fun and its result is used as the updated value of the matching key. If key cannot be matched the { key , default }-pair is inserted in the tree . Example iex&gt; t = new ( ) iex&gt; t = update ( t , &lt;&lt; 1 , 1 , 1 &gt;&gt; , 1 , fn x -&gt; x + 1 end ) iex&gt; t { 0 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 1 } ] , nil } iex&gt; t = update ( t , &lt;&lt; 1 , 1 , 1 , 0 &gt;&gt; , 1 , fn x -&gt; x + 1 end ) iex&gt; t { 0 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 2 } ] , nil } iex&gt; t = update ( t , &lt;&lt; 1 , 1 , 1 , 255 &gt;&gt; , 1 , fn x -&gt; x + 1 end ) iex&gt; t { 0 , [ { &lt;&lt; 1 , 1 , 1 &gt;&gt; , 3 } ] , nil }","ref":"Radix.html#update/4","title":"Radix.update/4","type":"function"},{"doc":"Returns all values from the radix tree . Example iex&gt; t = new ( [ ...&gt; { &lt;&lt; 1 , 1 , 1 , 0 :: 1 &gt;&gt; , &quot;1.1.1.0/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } , ...&gt; { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1.0/24&quot; } , ...&gt; { &lt;&lt; 3 &gt;&gt; , &quot;3.0.0.0/8&quot; } , ...&gt; ] ) iex&gt; iex&gt; # get values iex&gt; iex&gt; values ( t ) [ &quot;1.1.1.0/25&quot; , &quot;1.1.1.0/24&quot; , &quot;1.1.1.128/25&quot; , &quot;3.0.0.0/8&quot; ]","ref":"Radix.html#values/1","title":"Radix.values/1","type":"function"},{"doc":"Invokes fun on all (internal and leaf) nodes of the radix tree using either :inorder , :preorder or :postorder traversal. fun should have the signatures: ( acc/0 , tree/0 ) -&gt; acc/0 ( acc/0 , leaf/0 ) -&gt; acc/0 Note that leaf/0 might be nil. Example iex&gt; t = new ( [ { &lt;&lt; 1 &gt;&gt; , 1 } , { &lt;&lt; 2 &gt;&gt; , 2 } , { &lt;&lt; 3 &gt;&gt; , 3 } , { &lt;&lt; 128 &gt;&gt; , 128 } ] ) iex&gt; iex&gt; f = fn ...&gt; ( acc , { _bit , _left , _right } ) -&gt; acc ...&gt; ( acc , nil ) -&gt; acc ...&gt; ( acc , leaf ) -&gt; acc ++ Enum . map ( leaf , fn { _k , v } -&gt; v end ) ...&gt; end iex&gt; iex&gt; walk ( t , [ ] , f ) [ 1 , 2 , 3 , 128 ]","ref":"Radix.html#walk/4","title":"Radix.walk/4","type":"function"},{"doc":"A user supplied accumulator.","ref":"Radix.html#t:acc/0","title":"Radix.acc/0","type":"type"},{"doc":"A bitstring used as a key to index into the radix tree. During tree traversals, bit positions in the key are checked in order to decide whether to go left (0) or right (1). During these checks, bits beyond the current key's length always evaluate to 0.","ref":"Radix.html#t:key/0","title":"Radix.key/0","type":"type"},{"doc":"A radix leaf node. A leaf is either nil or a list of key,value-pairs sorted on key-length in descending order. All keys in a leaf have the other, shorter keys, as their prefix.","ref":"Radix.html#t:leaf/0","title":"Radix.leaf/0","type":"type"},{"doc":"An internal radix tree node. An internal node is a three element tuple: { bit , left , right }, where: bit is the bit position to check in a key left is a subtree with keys whose bit is 0 right is a subtree with keys whose bit is 1 The keys stored below any given internal node, all agree on the bits checked to arrive at that particular node. Branches in the tree are only created when storing a new key,value-pair in the tree whose key does not agree with the leaf found during traversal. This path-compression means not all bits in a key are checked while traversing the tree, only those which differentiate the keys stored below the current internal node. Hence, a final match is needed to ensure a correct match.","ref":"Radix.html#t:tree/0","title":"Radix.tree/0","type":"type"},{"doc":"Any value to be stored in the radix tree.","ref":"Radix.html#t:value/0","title":"Radix.value/0","type":"type"},{"doc":"RadixError provides information on error conditions that may occur.","ref":"RadixError.html","title":"RadixError","type":"exception"},{"doc":"","ref":"RadixError.html#exception/2","title":"RadixError.exception/2","type":"function"},{"doc":"Callback implementation for Exception.message/1 .","ref":"RadixError.html#message/1","title":"RadixError.message/1","type":"function"},{"doc":"An exception struct whose reason field contains a tuple with an error_id and the offending data.","ref":"RadixError.html#t:t/0","title":"RadixError.t/0","type":"type"},{"doc":"README A bitwise radix tree for prefix based matching on bitstring keys of any length. Radix provides a radix tree whose radius is 2, has path-compression and no one-way branching. Entries consist of {key, value}-pairs whose insertion/deletion is always based on an exact key-match. Retrieval can be either exact or is based on a longest prefix match.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"iex&gt; t = new ( ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1/24&quot; ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 , 0 :: 6 &gt;&gt; , &quot;1.1.1.0/30&quot; ) ...&gt; |&gt; put ( &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; ) ...&gt; |&gt; put ( &lt;&lt; 255 &gt;&gt; , &quot;255/8&quot; ) iex&gt; iex&gt; iex&gt; # longest prefix match iex&gt; iex&gt; lookup ( t , &lt;&lt; 1 , 1 , 1 , 255 &gt;&gt; ) { &lt;&lt; 1 , 1 , 1 , 1 :: 1 &gt;&gt; , &quot;1.1.1.128/25&quot; } iex&gt; iex&gt; iex&gt; # more specific matches (includes search key if present) iex&gt; iex&gt; more ( t , &lt;&lt; 1 , 1 , 1 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 1 , 0 :: size ( 6 ) &gt;&gt; , &quot;1.1.1.0/30&quot; } , { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1/24&quot; } , { &lt;&lt; 1 , 1 , 1 , 1 :: size ( 1 ) &gt;&gt; , &quot;1.1.1.128/25&quot; } ] iex&gt; iex&gt; iex&gt; # less specific matches (includes search key if present) iex&gt; iex&gt; less ( t , &lt;&lt; 1 , 1 , 1 , 3 &gt;&gt; ) [ { &lt;&lt; 1 , 1 , 1 , 0 :: size ( 6 ) &gt;&gt; , &quot;1.1.1.0/30&quot; } , { &lt;&lt; 1 , 1 , 1 &gt;&gt; , &quot;1.1.1/24&quot; } ] iex&gt; iex&gt; # exact match iex&gt; get ( t , &lt;&lt; 1 , 1 , 1 , 0 :: 6 &gt;&gt; ) { &lt;&lt; 1 , 1 , 1 , 0 :: 6 &gt;&gt; , &quot;1.1.1.0/30&quot; } iex&gt; iex&gt; get ( t , &lt;&lt; 1 , 1 , 1 , 0 &gt;&gt; ) nil iex&gt; iex&gt; dot ( t ) |&gt; ( &amp; File . write ( &quot;assets/readme.dot&quot; , &amp;1 ) ) . ( ) The radix tree above looks something like this: The tree is represented by two types of nodes: internal node , as a {bit, left, right} -tuple, and leaf node , which is either nil or a non-empty list of {key,value} -pairs The bit denotes the bit position to check in a key during a tree traversal, where 0 means go left and 1 means go right . A bit beyond a key 's length is considered to be 0 . Path-compression means not all bits are checked during tree traversal, only those that differentiate the keys stored below the current internal node. Hence, branches are formed as keys with different patterns are stored in the tree. The leaf node can have a list of {key, value} -pairs where all longer keys have all shorter keys as their prefix. In other words, they all agree on the bits that were checked to arrive at that node. The key is stored alongside the value since, due to path-compression, a final match is needed to ensure a correct match. Hence, retrieval functions return the {key, value} -pair, rather than just the value , since the stored key is not always equal to the given search key (e.g. when doing a longest prefix match). Since binaries are bitstrings too, they work as well: iex&gt; t = new ( [ { &quot;A.new&quot; , &quot;new&quot; } , { &quot;A.newer&quot; , &quot;newer&quot; } , { &quot;B.newest&quot; , &quot;newest&quot; } ] ) iex&gt; more ( t , &quot;A.&quot; ) |&gt; Enum . reverse ( ) [ { &quot;A.new&quot; , &quot;new&quot; } , { &quot;A.newer&quot; , &quot;newer&quot; } ] # iex&gt; lookup ( t , &quot;A.newest&quot; ) { &quot;A.new&quot; , &quot;new&quot; } # iex&gt; more ( t , &quot;C.&quot; ) [ ] Note: if Radix is used only for put / get operations, using a regular map is way faster.","ref":"readme.html#examples","title":"Overview - Examples","type":"extras"},{"doc":"The package can be installed by adding radix to your list of dependencies in mix.exs : def deps do [ { :radix , &quot;~&gt; 0.2.0&quot; } ] end Documentation can be found at https://hexdocs.pm/radix .","ref":"readme.html#installation","title":"Overview - Installation","type":"extras"},{"doc":"MIT License Copyright (c) 2021 hertogp Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"Changelog All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - [Unreleased]","type":"extras"},{"doc":"changed Radix.fetch/3 now can optionally use longest prefix match functions raise their own errors use assets subdir for images added Radix.empty?/1 , says if a tree is empty or not Radix.count/1 , traverses the tree and counts its entries Radix.merge/2 , merges tree2 into tree1 , overriding tree1 Radix.merge/3 , merges tree2 into tree1 , conflicts handled by fun Radix.take/3 , returns new tree with selected keys only Radix.pop/3 , removes key,value-pair and returns it with the new tree Radix.split/3 , split a radix tree into two trees changed","ref":"changelog.html#v0-2-0-2021-07-18","title":"Changelog - [v0.2.0] - 2021-07-18","type":"extras"},{"doc":"changed bad_xxx functions were meant to be private [v0.1.0] - 2021-06-21 Initial version.","ref":"changelog.html#v0-1-1-2021-06-21","title":"Changelog - [v0.1.1] - 2021-06-21","type":"extras"}]